<?php

/**
 * Class SMTP
 *
 * Features:
 * - supports SMTP and ESMTP servers
 * - password authentication (AUTH CRAM-MD5 OR AUTH LOGIN)
 * - SSL/TLS encryption
 *
 * Last changes:
 * 2010-06-28 Minor bug fixes
 * 2010-06-27 Created
 *
 * @author Valentin Schmidt
 * @version 0.1.2
 */
class SMTP{

var $host;
var $port;
var $username;
var $password;
var $localhost;

var $fp = false;
var $log = '';

# config defaults
var $timeout = 30;
var $prefer_auth_cram_md5 = true;

/**
 * Set SMTP configuration
 *
 * @access public
 * @param string $host for encrypted communication prepend 'ssl://' or 'tls://'
 * @param int $port
 * @param string $username optional
 * @param string $password optional
 * @param string $localhost optional, specify real (reverse lookup safe) hostname of current apache server to avoid spamassassin's FORGED_RCVD_HELO rating
 * @return void
 */
#public 
function setConfig($host='localhost', $port=25, $username=null, $password=null, $localhost=null){
	$this->host     = $host;
	$this->port     = $port;
	$this->username = $username;
	$this->password = $password;
	$this->localhost = is_null($localhost)?$_SERVER['SERVER_NAME']:$localhost;
}

/**
 * Send email
 *
 * @access public
 * @param string $to
 * @param string $subject encoded as $encoding
 * @param string $message encoded as $encoding
 * @param array $headers optional
 * @param string $encoding optional default = UTF-8
 * @return bool success
 */
#public 
function sendMail($to, $subject, $message, $headers=null, $encoding='UTF-8'){
	$this->log = '';
	mb_internal_encoding($encoding);
	$subject = mb_encode_mimeheader($subject, $encoding);
	list($to_address, $to_name) = $this->parseAddressString($to);
	if (!is_array($headers)) $headers=array();
	
	# DEFAULT HEADERS
	if (!isset($headers['From'])) $headers['From'] = @ini_get('sendmail_from');
	list($from_address, $from_name) = $this->parseAddressString($headers['From']);
	if (!isset($headers['X-Sender'])) $headers['X-Sender'] = "<$from_address>";
	if (!isset($headers['Return-Path'])) $headers['Return-Path'] = "<$from_address>";
	if (!isset($headers['Errors-To'])) $headers['Errors-To'] = "<$from_address>";
	if (!isset($headers['Content-Type'])) $headers['Content-Type'] = 'text/plain; charset='.$encoding;
	if (!isset($headers['X-Mailer'])) $headers['X-Mailer'] = 'PHP';
	if (!isset($headers['X-Priority'])) $headers['X-Priority'] = '3';
		
	$this->fp = fsockopen($this->host, $this->port, $errno, $errstr, $this->timeout);
	if ($this->fp===false){
		#throw new Exception("Could not connect to SMTP host ($errno : $errstr)", 1);
		return false;
	}
	$this->rcv();
	
	if(!empty($this->username) && !empty($this->password)){
		$result = $this->snd('EHLO '.$this->localhost, false); # 250
		
		# GET SERVER CAPABILITIES
		$capabilities = array();
		while(true){
			$str = fgets($this->fp, 1024);
			$this->log .= "RCV: $str";
			$str = substr($str, 0, -2);
			if (substr($str,3,1) != ' ') $capabilities[strtok(substr($str, 4), ' ')] = $str;
			else break;
		}
		# CHECK RESULT CODE
		$result = explode(' ', $str);
		if ($result[0]!='250'){
			fclose($this->fp);
			return false;
		}
	
		if ($this->prefer_auth_cram_md5&&isset($capabilities['AUTH'])&&strpos($capabilities['AUTH'], 'CRAM-MD5')!==false){
			$ok = $this->auth_cram_md5($this->username, $this->password);
		}else{
			$ok = $this->auth_login($this->username, $this->password);
		}
		
	}else{
		$result = $this->snd('HELO '.$this->localhost); # 250
		if ($result[0]!='250'){
			fclose($this->fp);
			return false;
		}
	}

	$ok = true;
	$this->snd("MAIL FROM: $from_address"); # 250
	$this->snd("RCPT TO: $to_address"); # 250
	$this->snd('DATA'); # 354
	
	$this->snd('Message-ID: <'.md5(uniqid(rand())).'@'.$this->localhost.'>', false);
	$this->snd("Subject: $subject", false);
	$this->snd("To: $to_name  <$to_address>", false);
	foreach ($headers as $header=>$value){
		$this->snd("$header: $value", false);
	}
	$this->snd('', false);
	$this->snd(stripslashes($message), false);
	$result = $this->snd('.'); # 250
	$ok = ($result[0]=='250');
	$this->snd('RSET'); # 250
	$this->snd('QUIT'); # 221
	fclose($this->fp);
	return $ok;
}

/**
 * Return SMTP-PROTOCOL
 *
 * @access public
 * @return string SMTP-PROTOCOL
 */
#public 
function getLog(){
	return $this->log;
}

/**
 * Send SMTP command, (optionally) receive result and return numeric result code
 *
 * @access private
 * @param string $send
 * @param bool $receiveFlag optional, default = true
 * @return void|array
 */
#private
function snd($send, $receiveFlag=true){
	fputs($this->fp, $send."\r\n");
	$this->log .= "SND: $send\n";
	if ($receiveFlag) return $this->rcv();
}

/**
 * Receive SMTP result 
 *
 * @access private
 * @return array result chunks 
 */
#private
function rcv(){	
	$str = fgets($this->fp, 1024);
	$this->log .= "RCV: $str";
	return explode(' ', $str);
}

/**
 * AUTH LOGIN
 *
 * @access private
 * @return bool success
 */
#private
function auth_login($username, $password){
	$this->snd('AUTH LOGIN'); # 334
	$this->snd(base64_encode($username)); # 334
	$result = $this->snd(base64_encode($password)); # 235
	return ($result[0]=='235');
}

/**
 * AUTH CRAM-MD5
 *
 * @access private
 * @return bool success
 */
#private
function auth_cram_md5($username, $password){
	$result = $this->snd('AUTH CRAM-MD5');
	$challenge = base64_decode($result[1]); 
	$key = $password;
	if (strlen($key) > 64) {
		$key = pack('H32', md5($key));
	} elseif (strlen($key) < 64) {
		$key = str_pad($key, 64, chr(0));
	}
	$k_ipad = substr($key, 0, 64) ^ str_repeat(chr(0x36), 64);
	$k_opad = substr($key, 0, 64) ^ str_repeat(chr(0x5C), 64);
	$inner = pack('H32', md5($k_ipad . $challenge));
	$digest = md5($k_opad . $inner);
	$result = $this->snd(base64_encode($username.' '.$digest)); # 235
	return ($result[0]=='235');
}

/**
 * Parses simple email address string (e.g. "name <mbox@domain.com>")
 *
 * @access private
 * @param string $str
 * @return array (address, name)
 * @todo use regular expression?
 */
#private
function parseAddressString($str){
	$tmp = explode('<', $str);
	if (count($tmp)>1){
		$name = trim($tmp[0]);
		$address = trim(substr($tmp[1],0,-1));
	}else{
		$name = '';
		$address = trim($str);
	}
	return array($address, $name);
}

}//END CLASS

?>